<?php
// getrescent plugin - Show comments into specified (another) page
//					   (readonly pcomment)
//
// Usage: #getrecent([page][,max][,options])
//
//   page -- An another page-name that holds comments
//           (default:PLUGIN_PCOMMENT_PAGE)
//   max  -- Max number of recent comments to show
//           (0:Show all, default:PLUGIN_PCOMMENT_NUM_COMMENTS)
//
// Options:
//   above -- chronological order
//   below -- reverse order

// Default recording page name (%s = $vars['page'] = original page name)
define('PLUGIN_GETRECENT_PAGE', _('[[Comments/%s]]'));

define('PLUGIN_GETRECENT_NUM_COMMENTS',     10); // Default 'latest N posts'
define('PLUGIN_GETRECENT_DIRECTION_DEFAULT', 1); // 1: above 0: below

function plugin_getrecent_convert()
{
	global $vars;
//	global $_pcmt_messages;
	$_pcmt_messages = array(
		'msg_recent'     => _('Show recent %d topics.'),
		'msg_all'        => _('Go to the full page.'),
		'msg_none'       => _('No topic.'),
		'err_pagename'   => _('[[%s]] : not a valid page name.'),
	);

	$ret = '';

	$params = array(
		'noname'=>FALSE,
		'nodate'=>FALSE,
		'below' =>FALSE,
		'above' =>FALSE,
		'reply' =>FALSE,
		'_args' =>array()
	);

	// BugTrack2/106: Only variables can be passed by reference from PHP 5.0.5
	$args = func_get_args(); // with array_walk()
	array_walk($args, 'plugin_getrecent_check_arg', & $params);

	$vars_page = isset($vars['page']) ? $vars['page'] : '';
	$page  = (isset($params['_args'][0]) && $params['_args'][0] != '') ? $params['_args'][0] :
		sprintf(PLUGIN_GETRECENT_PAGE, strip_bracket($vars_page));
	$count = (isset($params['_args'][1]) && $params['_args'][1] != '') ? $params['_args'][1] : 0;
	if ($count == 0 && $count !== '0')
		$count = PLUGIN_GETRECENT_NUM_COMMENTS;

	$_page = get_fullname(strip_bracket($page), $vars_page);
	if (!is_pagename($_page))
		return sprintf($_pcmt_messages['err_pagename'], htmlspecialchars($_page));

	$dir = PLUGIN_GETRECENT_DIRECTION_DEFAULT;
	if ($params['below']) {
		$dir = 0;
	} elseif ($params['above']) {
		$dir = 1;
	}

	list($comments, $digest) = plugin_getrecent_get_comments($_page, $count, $dir);

	if (! is_page($_page)) {
		$link   = make_pagelink($_page);
		$recent = $_pcmt_messages['msg_none'];
	} else {
		$msg    = ($_pcmt_messages['msg_all'] != '') ? $_pcmt_messages['msg_all'] : $_page;
		$link   = make_pagelink($_page, $msg);
		$recent = ! empty($count) ? sprintf($_pcmt_messages['msg_recent'], $count) : '';
	}

	return '<div>' .
		'<p>' . $recent . ' ' . $link . '</p>' . "\n" .
		$comments . "\n" .
		'</div>' . "\n";
}

function plugin_getrecent_check_arg($val, $key, & $params)
{
	if ($val != '') {
		$l_val = strtolower($val);
		foreach (array_keys($params) as $key) {
			if (strpos($key, $l_val) === 0) {
				$params[$key] = TRUE;
				return;
			}
		}
	}

	$params['_args'][] = $val;
}

function plugin_getrecent_get_comments($page, $count, $dir)
{
	if (! check_readable($page, false, false))
		return array(str_replace('$1', $page, _('Due to the blocking, no comments could be read from  $1 at all.')));

	// $reply = (! PKWK_READONLY && $reply); // Suprress radio-buttons
	$reply = (! auth::check_role('readonly') && $reply); // Suprress radio-buttons

	$data = get_source($page);
	$data = preg_replace('/^#getrecent\(?.*/i', '', $data);	// Avoid eternal recurse

	if (! is_array($data)) return array('', 0);

	$digest = md5(join('', $data));

	// Get latest N comments
	$num  = $cnt     = 0;
	$cmts = $matches = array();
	if ($dir) $data = array_reverse($data);
	foreach ($data as $line) {
		if ($count > 0 && $dir && $cnt == $count) break;
		if (preg_match('/^(\-{1,2})(?!\-)(.+)$/', $line, $matches)) {
			if ($count > 0 && strlen($matches[1]) == 1 && ++$cnt > $count) break;
		}
		$cmts[] = $line;
	}
	$data = $cmts;
	if ($dir) $data = array_reverse($data);
	unset($cmts, $matches);

	// Remove lines before comments
	while (! empty($data) && substr($data[0], 0, 1) != '-')
		array_shift($data);

	$comments = convert_html($data);
	unset($data);

	return array($comments, $digest);
}
?>
